use StatisticsTest;
go

-- DBCC
dbcc show_statistics(T1,IxT1_x)

-- sp_helpstats; not very useful => depricated
exec sp_helpstats 'dbo.T1'


-- DMV 1: general
select object_name(object_id) as table_name 
      ,name as stats_name 
      ,stats_date(object_id, stats_id) as last_update
  from sys.stats
 where objectproperty(object_id, 'IsUserTable') = 1
 order by last_update
go


-- DMV 2: including column info
with rows_per_table as
 (
  select id,rowcnt
    from sys.sysindexes
   where indid<2  
 )
select object_schema_name(s.object_id) + '.' + object_name(s.object_id) as table_name
      ,s.name as stats_name
      ,s.auto_created as is_auto_created
      ,s.user_created as is_user_created
      ,stuff((select ','+col_name(object_id,column_id)
                from sys.stats_columns as sc
               where sc.object_id = s.object_id
                 and sc.stats_id = s.stats_id
               order by stats_column_id
                 for xml path('')), 1, 1, '') as stats_cols
      ,stats_date(s.object_id,s.stats_id) as last_update
      ,r.rowcnt as total_rows
      ,si.rowmodctr as changed_rows
      ,cast(case
              when r.rowcnt = 0 then 0
              else 100.*si.rowmodctr/r.rowcnt
            end as decimal(18,1)) as percent_changed
  from sys.stats as s
       inner join sys.sysindexes as si
               on si.id = s.object_id
              and si.indid = s.stats_id 
       inner join rows_per_table as r
               on r.id = s.object_id
              and r.id = si.id
 where objectproperty(s.object_id, 'IsUserTable')=1
 order by table_name, stats_name
go

