use StatisticsTest
go

-- Problem 1: Local variable => no histogram utilization
declare @x int
set @x = 2000
select c1,c2 from T0
 where c1 = @x
 

-- Solution 1: SP
if (object_id('getT0Values', 'P') is not null)
  drop procedure getT0Values
go  
create procedure getT0Values(@x int) as
 select c1,c2 from T0
  where c1 = @x
go
exec getT0Values 2000
go

-- Solution 2: dynamic SQL
declare @x int
       ,@cmd nvarchar(300)
set @x = 2000
set @cmd = 'select c1,c2 from T0 where c1=' 
               + cast(@x as nvarchar(8))
exec (@cmd)
go

-- Solution 3: forced parameterization
exec sp_executesql N'select c1,c2 from T0 where c1=@x'
                  ,N'@x int'
                  ,@x = 2000
go

----------------------------------------------------------
-- Problem 2: Non-foldable expressions

-- Foldable expressions
select c1,c2 from T0
 where c1*10 = 10000

select c1,c2 from T0
 where c1+100 = 2100

 
-- Non-foldable expression
select c1,c2 from T0
 where abs(c1) = 2000

-- Calculated column
alter table T0 add AbsC1 as abs(c1)
select c1,c2 from T0
 where AbsC1 = 2000
go

-- Problem 3: Paramaterization issues

-- Modifying parameter values in SPs
if (object_id('getT0Values', 'P') is not null)
  drop procedure getT0Values
go  
create procedure getT0Values(@x int) as
 -- Don't do this!
 set @x = @x * 2
 select c1,c2 from T0
  where c1 = @x
go
exec getT0Values 1000
go


-- First parameter-set determines plan
if (object_id('getT0Values', 'P') is not null)
  drop procedure getT0Values
go  
create procedure getT0Values(@x int) as
 select c1,c2 from T0
  where c1 = @x
go
exec getT0Values 1000
go
exec getT0Values 2000
go

-- Possible solution
exec getT0Values 2000 with recompile
go
