use StatisticsTest;
go

-- Create Product table
if (object_id('Product', 'U') is not null)
  drop table Product 
go
create table Product
 ( 
   ProductId int identity(1,1) not null
  ,ListPrice decimal(8,2) not null 
  ,LastUpdate date not null default current_timestamp
  ,filler nchar(500) not null default '#'
 )
go
alter table Product add constraint PK_Product
  primary key clustered (ProductId)

create nonclustered index ix_Product_LastUpdate on Product(LastUpdate)
go

-- Insert 500000 products and update ass existing statistics
insert Product(LastUpdate, ListPrice)
 select dateadd(day, abs(checksum(newid())) % 3250,'20000101')
       ,0.01*(abs(checksum(newid())) % 20000)
   from Numbers where n <= 500000
go
update statistics Product with fullscan 
go

-- Now, insert 100000 more products
insert Product(LastUpdate,ListPrice)
 select '20100101', 100 from Numbers where n <= 100000   
go

-- Problem: stale statistics
dbcc freeproccache
set statistics io on
select * from Product where LastUpdate = '20100101'

-- Solution 1: Index hint
select * from Product with (index=0)
 where LastUpdate = '20100101'
go

-- Solution 2: update statistics
update statistics Product with fullscan 
go

-- Solution 3: index rebuild. 
-- will also perform update statistics
alter index ix_Product_LastUpdate on dbo.Product rebuild 
go
