use StatisticsTest
go

-- Rental cars
if (object_id('RentalCar', 'U') is not null)
  drop table RentalCar
go  
create table RentalCar
 (
  RentalCarID int not null identity(1,1)
      primary key clustered
 ,CarType nvarchar(20) not null
 ,DailyRate decimal(6,2)
 ,MoreColumns nchar(200) not null default '#'
 )
go

create nonclustered index Ix_RentalCar_CarType_DailyRate
   on RentalCar(CarType, DailyRate)
go

-- Insert 100000 cars with 4 different types
-- 25000 cars of each type
-- Diferent price-segment for every type
with CarTypes(minRate, maxRate, carType) as
 (
  select 20, 39, 'Compact'
   union all select 40, 59, 'Medium'
   union all select 60, 89, 'FullSize'
   union all select 90, 140, 'Luxory'
 )
insert RentalCar(CarType, DailyRate)
  select carType, minRate+abs(checksum(newid()))%(maxRate-minRate)
    from CarTypes
         inner join Numbers on n <= 25000
go
-- Update existing statistics
update statistics RentalCar with fullscan
go

---------------------------------------------------
-- Check
select * from RentalCar
go


set statistics io on

-- Check this
select * from RentalCar
 where CarType='Luxory'
   and DailyRate < 90


-- Reason for Table Scan
-- See estimated row counts
select count(*) from RentalCar
 where CarType='Luxory'

select count(*) from RentalCar
 where DailyRate < 90

-- Density fr CarType and DailyRate 
select 25000./100000
      ,63586./100000
-- Estimated Rows
select 100000* 0.25*0.63586
---------------------------------------------------

-- Solution 1: Index hint
select * from RentalCar with (index=Ix_RentalCar_CarType_DailyRate)
 where CarType='Luxory'
   and DailyRate < 90

select * from RentalCar with (forceseek)
 where CarType='Luxory'
   and DailyRate < 90






-- Solution 2: Filtered index
create nonclustered index Ix_RentalCar_LuxoryCar_DailyRate
   on RentalCar(DailyRate)
where Cartype='Luxory'
go
  
select * from RentalCar
 where CarType='Luxory'
   and DailyRate < 90






-- Solution 3: Filtered statistics  
drop index Ix_RentalCar_LuxoryCar_DailyRate on RentalCar
go  

create statistics sf on RentalCar(DailyRate)
 where CarType='Luxory'
go

select * from RentalCar
 where CarType='Luxory'
   and DailyRate < 90





-- Solution 4: Covering index 
drop statistics RentalCar.sf 
go
create index IxRentalCar_Covering
  on RentalCar(CarType, DailyRate)
 include (MoreColumns)
go

select * from RentalCar
 where CarType='Luxory'
   and DailyRate < 90

